/******************************************************************************
SBRetriggerableCreatureFactory, Version 4.

This is a CreatureFactory that can be retriggered after a spawn cycle.
*MaxLivingItems* is the maximum number of *Prototype* that can exist at any
time.
*MaxItems* and *Capacity* are as usual for one spawn cycle.

Set *bOnlyTriggerable* to TRUE if you want to disable *Touch* and *UnTouch*.


2016-02-19, SeriousBarbie AT googlemail MAKEADOT com
******************************************************************************/

class SBRetriggerableCreatureFactory extends CreatureFactory;


var(ThingFactory) int MaxLivingItems;
var(ThingFactory) bool bOnlyTriggerable; // Factory can only activated by Trigger event

var int fCurrentItemsCount; // internal counter for total living prototypes
var int FactoryCapacity; // original value of *CreatureFactory.Capacity*


function PostBeginPlay() {
	Super.PostBeginPlay();
	if (MaxLivingItems < MaxItems)
	{
		warn("MaxLivingItems may not be lower than MaxItems - MaxLivingItems set to MaxItems=" $ MaxItems);
		MaxLivingItems = MaxItems;
	}
	FactoryCapacity = Capacity;
	if (bOnlyTriggerable)
	{
		Disable('UnTouch');
		Disable('Touch');
	}

}


Auto State Waiting {

	function Trigger(actor Other, pawn EventInstigator) {

		if (Other.class == prototype && Other.Tag == ItemTag)
		{
			fCurrentItemsCount--;
			return;
		}

		if (fCurrentItemsCount >= MaxLivingItems)
			return;
		Super.Trigger(Other, EventInstigator);

	}

	function Touch(Actor Other) {
	local pawn otherpawn;

		if (bOnlyTriggerable)
			return;
		otherpawn = Pawn(Other);
		if ( (otherpawn != None) && ( ! bOnlyPlayerTouched || otherpawn.bIsPlayer) )
			if ( ! (Other.class == prototype && Other.Tag == itemtag)) // don't let our own creatures trigger it
				Trigger(other, otherPawn);
	}

}


State Spawning {


	function Trigger(actor Other, pawn EventInstigator) {

		if (Other.class == prototype)
			fCurrentItemsCount--;
		Super.Trigger(Other, EventInstigator);
	}


	function bool trySpawn(int start, int end){
	local bool result;

		if (fCurrentItemsCount >= MaxLivingItems)
			Return false;
		result = Super.trySpawn(start, end);
		if (Result)
			fCurrentItemsCount++;
		Return result;
	}

	function UnTouch(Actor Other) {
		if (bOnlyTriggerable)
			return;
		else
			Super.UnTouch(Other);
	}

}


state Finished {

	function BeginState() {
		numitems = 0;
		Capacity = FactoryCapacity;
		GotoState('Waiting');
	}
}


defaultproperties {
	bOnlyTriggerable=true
	MaxLivingItems=3
}
